function [P_kp, gamma_k] = map_H(P,A,C,Q,R,P_bar,delta)
% [P_kp, gamma_k] = map_H(P,A,C,Q,R,P_bar,delta)
% Function outputs P(k+1), gamma(k).
%
% Map H: (eq. (21) in paper mentioned below)
%
%   P(k+1) = H(P(k)) 
%       = A P(k) A^T + Q - A P(k) (C(P(k)))^T 
%          * ( C(P(k)) P(k) (C(P(k)))^T + R(P(k)) )^{-1} 
%          * C(P(k)) P(k) A^T
%
% Matrices C(P(k)) and R(P(k)) are constructed according to the number of 
% available measurements, that is, according the eq. (11).
% The decision rule (17), for using a measurement y_j is:
%
%   use y_j   <=>   C_j (P(k)-P_bar) C_j^T >= \delta_j.
% 
% C(P(k)) is then constructed by stacking all rows that correspond to used
% measurements; and R(P(k)) has the measurement noise variances 
% corresponding to the used measurements on its diagonal.
%
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Event-Based State Estimation with Variance-Based 
% Triggering", IEEE Transactions on Automatic Control, Special Issue on 
% Control of Cyber-Physical Systems.
%
% Copyright:
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% strimpe(at)tuebingen.mpg.de
% March 2014
%

% Revision history
% [Dec 12, ST]   first version
% [Dec 13, ST]   adapt equation numbers etc. to revised vrsion of the paper
% [10.03.14, ST] adapt to revised version of the paper


%% Prep
M = size(C,1);      % total number of measurements


%% Contruct output matrices according to decision rule
% Construct matrices C(P(k)) and R(P(k)) according to equation (11).

% start with empty matrices
Ck = [];        
Rk = [];

% Check for each measurement individually.
for j=1:M
    % Check sending decision (17):
    gamma_k(j) = ( C(j,:)*(P-P_bar)*(C(j,:))' >= delta(j) );
    if (gamma_k(j))
        % The measurement y(j) is used.
        Ck = [Ck; C(j,:)];          % append row
        Rk = blkdiag(Rk,R(j,j));    % append diagonal element
    end;
end;


%% Evaluate H
% Evaluate the map H in (21).

if ~isempty(Ck)
    % Measurement vector not empty; use Ck, Rk for update step.
    P_kp = A*P*A'+Q - A*P*Ck'*inv(Ck*P*Ck'+Rk)*Ck*P*A';
else
    % Measurement vector empty; do prediction part only.
    P_kp = A*P*A'+Q;
end;

